@@ -9,6 +9,11 @@ setClass("XStringSet",
     prototype(elementType="XString")
 )
 
+### This tells some operations (e.g. pcompare() and narrow()) to ignore the
+### list-like nature of XStringSet derivatives i.e. to treat their list
+### elements as atoms.
+setMethod("pcompareRecursively", "XStringSet", function(x) FALSE)
+
 ### The concrete XStringSet subclasses below have no additional slots.
 setClass("BStringSet",
     contains="XStringSet",
@@ -91,14 +96,15 @@ setMethod("nchar", "XStringSet",
 ### Methods for XStringSet objects are inherited from the XVectorList class.
 ###
 
-setMethod("narrow", "character",
-    function(x, start=NA, end=NA, width=NA, use.names=TRUE)
+### The default "narrow" method calls windows() so we only need to implement
+### a "windows" method for character vectors to make narrow() also work on
+### them.
+setMethod("windows", "character",
+    function(x, start=NA, end=NA, width=NA)
     {
-        if (!normargUseNames(use.names))
-            names(x) <- NULL
         x_width <- width(x)
-        solved_SEW <- solveUserSEW(x_width, start=start, end=end, width=width)
-        substr(x, start=start(solved_SEW), stop=end(solved_SEW))
+        ir <- solveUserSEW(x_width, start=start, end=end, width=width)
+        substr(x, start=start(ir), stop=end(ir))
     }
 )
 
