@@ -41,12 +41,15 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(XString_xscat, 1),
 	CALLMETHOD_DEF(XStringSet_xscat, 1),
 
-/* XStringSet_io.c */
+/* read_fasta_files.c */
+	CALLMETHOD_DEF(read_fasta_files, 7),
 	CALLMETHOD_DEF(fasta_index, 5),
-	CALLMETHOD_DEF(read_XStringSet_from_fasta_blocks, 6),
+	CALLMETHOD_DEF(read_fasta_blocks, 6),
 	CALLMETHOD_DEF(write_XStringSet_to_fasta, 4),
+
+/* read_fastq_files.c */
 	CALLMETHOD_DEF(fastq_seqlengths, 4),
-	CALLMETHOD_DEF(read_XStringSet_from_fastq, 8),
+	CALLMETHOD_DEF(read_fastq_files, 8),
 	CALLMETHOD_DEF(write_XStringSet_to_fastq, 4),
 
 /* letter_frequency.c */
